package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.dto.ShelfDto;
import com.junmp.jyzb.api.bean.query.ShelfReq;
import com.junmp.jyzb.api.bean.req.UpdateShelfReq;
import com.junmp.jyzb.api.exception.enums.ShelfExceptionEnum;
import com.junmp.jyzb.api.exception.enums.WarehouseExceptionEnum;
import com.junmp.jyzb.entity.Shelf;
import com.junmp.jyzb.entity.Warehouse;
import com.junmp.jyzb.service.WarehouseService;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.junmp.jyzb.mapper.ShelfMapper;
import com.junmp.jyzb.service.ShelfService;

import java.util.ArrayList;
import java.util.List;

@Service
public class ShelfServiceImpl extends ServiceImpl<ShelfMapper, Shelf> implements ShelfService {

    @Resource
    private WarehouseService warehouseService;

    @Override
    public String AddShelf(UpdateShelfReq req) {
        //判断仓库是否存在
        warehouseService.WarehoustExist(req.getWarehouseId());
        Shelf shelf = new Shelf();
        BeanPlusUtil.copyProperties(req,shelf);
        save(shelf);
        return shelf.getShelfId();
    }

    @Override
    public boolean UpdateShelf(UpdateShelfReq req) {
        //判断货架id是否正确
        Shelf shelf = this.QueryShelf(req);
        BeanPlusUtil.copyProperties(req, shelf);
        return this.updateById(shelf);
    }

    @Override
    public boolean DeleteShelf(UpdateShelfReq req) {
        QueryShelf(req);
        return removeById(req.getShelfId());
    }

    @Override
    public List<ShelfDto> GetShelfList(ShelfReq req) {
        //判断仓库id是否存在
        warehouseService.WarehoustExist(req.getWarehouseId());
        List<Shelf> list = list(new LambdaQueryWrapper<Shelf>()
                .eq(Shelf::getWarehouseId,req.getWarehouseId()));
        List<ShelfDto> shelfDtoList=new ArrayList<>();
        for (Shelf shelf:list) {
            ShelfDto shelfDto = new ShelfDto();
            BeanPlusUtil.copyProperties(shelf,shelfDto);
            shelfDtoList.add(shelfDto);
        }
        return shelfDtoList;
    }

    /**
     * 判断货架id是否存在
     * @param req
     * @return
     */
    private Shelf QueryShelf(UpdateShelfReq req) {
        Shelf shelf = this.getById(req.getShelfId());
        if (ObjectUtil.isNull(shelf)) {
            throw new ServiceException(ShelfExceptionEnum.SHELF_NOT_EXIST);
        }
        return shelf;
    }

}
